import { Omit } from './type-utilities';
export declare type Action<T extends string, P extends object = {}> = {
    readonly type: T;
} & P;
export interface Payload<P = {}> {
    readonly payload: P;
}
export declare type TypeOnlyAction<T extends string> = Action<T>;
export declare type PropsAction<T extends string, P extends object = {}> = Action<T, P>;
export declare type PayloadAction<T extends string, P = {}> = Action<T, Payload<P>>;
export declare enum ActionCreatorType {
    TYPE_ONLY = "TYPE_ONLY",
    PROPS = "PROPS",
    PAYLOAD = "PAYLOAD"
}
export interface ActionCreator<T extends string, P extends object = {}> {
    readonly __tsdux_action__: ActionCreatorType;
    readonly type: T;
    readonly action: Action<T, P>;
    create(...args: Array<never>): Action<T, P>;
}
export interface TypeOnlyActionCreatorProperties<T extends string> extends ActionCreator<T> {
    readonly __tsdux_action__: ActionCreatorType.TYPE_ONLY;
    create(): TypeOnlyAction<T>;
}
export interface TypeOnlyActionCreator<T extends string> extends TypeOnlyActionCreatorProperties<T> {
    (): TypeOnlyAction<T>;
}
export interface PropsActionCreatorProperties<T extends string, P extends object = {}> extends ActionCreator<T, P> {
    readonly __tsdux_action__: ActionCreatorType.PROPS;
    create(props: P): PropsAction<T, P>;
}
export interface PropsActionCreator<T extends string, P extends object = {}> extends PropsActionCreatorProperties<T, P> {
    (props: P): PropsAction<T, P>;
}
export interface PayloadActionCreatorProperties<T extends string, P = {}> extends ActionCreator<T, Payload<P>> {
    readonly __tsdux_action__: ActionCreatorType.PAYLOAD;
    create(payload: P): PayloadAction<T, P>;
}
export interface PayloadActionCreator<T extends string, P = {}> extends PayloadActionCreatorProperties<T, P> {
    (payload: P): PayloadAction<T, P>;
}
export declare enum ActionOptType {
    PROPS = "PROPS",
    PAYLOAD = "PAYLOAD"
}
export interface PropsOpt<P extends object = {}> {
    readonly __tsdux_opt__: ActionOptType.PROPS;
    readonly props: P;
}
export interface PayloadOpt<P = {}> {
    readonly __tsdux_opt__: ActionOptType.PAYLOAD;
    readonly payload: P;
}
export declare function action<T extends string>(type: T): TypeOnlyActionCreator<T>;
export declare function action<T extends string, P extends object = {}>(type: T, p: PropsOpt<P>): PropsActionCreator<T, P>;
export declare function action<T extends string, P = {}>(type: T, p: PayloadOpt<P>): PayloadActionCreator<T, P>;
export declare function props<P extends object = {}>(): PropsOpt<Omit<P, 'type'>>;
export declare function payload<P = {}>(): PayloadOpt<P>;
